# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.views.generic import (
    TemplateView,
)
from cookie_consent.util import get_cookie_value_from_request


class TestPageView(TemplateView):
    template_name = "test_page.html"

    def get(self, request, *args, **kwargs):
        response = super(TestPageView, self).get(request, *args, **kwargs)
        if get_cookie_value_from_request(request, "optional") is True:
            val = "optional cookie set from django"
            response.set_cookie("optional_test_cookie", val)
        return response



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.conf import settings  # NOQA

from appconf import AppConf


class CookieConsentConf(AppConf):
    NAME = b"cookie_consent"
    MAX_AGE = 60 * 60 * 24 * 365 * 1  # 1 year
    DECLINE = "-1"

    ENABLED = True

    OPT_OUT = False

    CACHE_BACKEND = "default"



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

try:
    from django.urls import reverse
except ImportError:
    from django.core.urlresolvers import reverse
from django.test import (
    TestCase,
)

from cookie_consent.models import (
    Cookie,
    CookieGroup,
    LogItem,
    ACTION_ACCEPTED,
    ACTION_DECLINED,
)


class IntegrationTest(TestCase):

    def setUp(self):
        self.cookie_group = CookieGroup.objects.create(
            varname="optional",
            name="Optional",
        )
        self.cookie = Cookie.objects.create(
            cookiegroup=self.cookie_group,
            name="foo",
        )

    def test_cookies_view(self):
        response = self.client.get(reverse('cookie_consent_cookie_group_list'))
        self.assertEqual(response.status_code, 200)
        self.assertContains(
            response,
            '<input type="submit" value="Accept">')
        self.assertContains(
            response,
            '<input type="submit" value="Decline">')

    def test_accept_cookie(self):
        response = self.client.post(reverse('cookie_consent_accept',
                                            kwargs={"varname": "optional"}),
                                    follow=True)
        self.assertEqual(response.status_code, 200)
        self.assertContains(
            response,
            '<span class="cookie-consent-accepted">Accepted</span>')

    def test_accept_cookie_ajax(self):
        response = self.client.post(reverse('cookie_consent_accept',
                                            kwargs={"varname": "optional"}),
                                    HTTP_X_REQUESTED_WITH='XMLHttpRequest')
        self.assertEqual(response.status_code, 200)

    def test_decline_cookie(self):
        response = self.client.post(reverse('cookie_consent_decline',
                                            kwargs={"varname": "optional"}),
                                    follow=True)
        self.assertEqual(response.status_code, 200)
        self.assertContains(
            response,
            '<span class="cookie-consent-declined">Declined</span>')

    def test_decline_cookie_ajax(self):
        response = self.client.delete(reverse('cookie_consent_decline',
                                              kwargs={"varname": "optional"}),
                                      HTTP_X_REQUESTED_WITH='XMLHttpRequest')
        self.assertEqual(response.status_code, 200)

    def test_cookies(self):
        CookieGroup.objects.create(
            varname="social",
            name="Social",
        )
        Cookie.objects.create(
            cookiegroup=self.cookie_group,
            name="optional_test_cookie",
        )

        url = reverse('test_page')
        response = self.client.get(url)
        self.assertContains(response,
                            '"optional" cookies not accepted or declined')

        response = self.client.post(reverse('cookie_consent_accept',
                                            kwargs={"varname": "optional"}))
        response = self.client.get(url)
        self.assertContains(response, '"optional" cookies accepted')
        self.assertEqual(self.client.cookies.get('optional_test_cookie').value,
                         "optional cookie set from django")

        response = self.client.post(reverse('cookie_consent_decline',
                                            kwargs={"varname": "optional"}))
        response = self.client.get(url)
        self.assertContains(response, '"optional" cookies declined')
        #test client returns cookie with value of ''
        #self.assertIsNone(self.client.cookies.get('optional_test_cookie'))
        self.assertFalse(self.client.cookies.get('optional_test_cookie').value)

    def test_logging(self):
        self.client.post(reverse('cookie_consent_accept',
                                 kwargs={"varname": "optional"}))
        log_items = LogItem.objects.filter(
            cookiegroup=self.cookie_group,
            version=self.cookie_group.get_version(),
            action=ACTION_ACCEPTED)
        self.assertEqual(log_items.count(), 1)

        self.client.delete(reverse('cookie_consent_decline',
                                   kwargs={"varname": "optional"}))
        log_items = LogItem.objects.filter(
            cookiegroup=self.cookie_group,
            version=self.cookie_group.get_version(),
            action=ACTION_DECLINED)
        self.assertEqual(log_items.count(), 1)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.contrib import admin

from .models import (
    Cookie,
    CookieGroup,
    LogItem,
)


class CookieAdmin(admin.ModelAdmin):
    list_display = ('varname', 'name', 'cookiegroup', 'path', 'domain',
                    'get_version')
    search_fields = ('name', 'domain', 'cookiegroup__varname',
                     'cookiegroup__name')
    readonly_fields = ('varname',)
    list_filter = ('cookiegroup',)


class CookieGroupAdmin(admin.ModelAdmin):
    list_display = ('varname', 'name', 'is_required', 'is_deletable',
                    'get_version')
    search_fields = ('varname', 'name',)
    list_filter = ('is_required', 'is_deletable',)


class LogItemAdmin(admin.ModelAdmin):
    list_display = ('action', 'cookiegroup', 'version', 'created')
    list_filter = ('action', 'cookiegroup')
    readonly_fields = ('action', 'cookiegroup', 'version', 'created')
    date_hierarchy = 'created'


admin.site.register(Cookie, CookieAdmin)
admin.site.register(CookieGroup, CookieGroupAdmin)
admin.site.register(LogItem, LogItemAdmin)



# Generated by Django 2.1 on 2019-02-08 14:16

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('cookie_consent', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='LogItem',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('action', models.IntegerField(choices=[(-1, 'Declined'), (1, 'Accepted')], verbose_name='Action')),
                ('version', models.CharField(max_length=32, verbose_name='Version')),
                ('created', models.DateTimeField(auto_now_add=True, verbose_name='Created')),
                ('cookiegroup', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='cookie_consent.CookieGroup', verbose_name='Cookie Group')),
            ],
            options={
                'verbose_name': 'Log item',
                'verbose_name_plural': 'Log items',
                'ordering': ['-created'],
            },
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals
try:
    from django.urls import reverse
except ImportError:
    from django.core.urlresolvers import reverse
from django.http import HttpResponseRedirect, HttpResponse
from django.views.generic import (
    ListView,
    View,
)

from .models import (
    CookieGroup,
)
from .util import (
    accept_cookies,
    decline_cookies,
)


class CookieGroupListView(ListView):
    """
    Display all cookies.
    """
    model = CookieGroup


class CookieGroupBaseProcessView(View):

    def get_success_url(self):
        if hasattr(self.request, 'next'):
            url = self.request.get("next")
        else:
            url = reverse('cookie_consent_cookie_group_list')
        return url

    def process(self, request, response, varname):
        raise NotImplementedError()

    def post(self, request, *args, **kwargs):
        varname = kwargs.get('varname', None)
        if request.is_ajax():
            response = HttpResponse()
        else:
            response = HttpResponseRedirect(self.get_success_url())
        self.process(request, response, varname)
        return response


class CookieGroupAcceptView(CookieGroupBaseProcessView):
    """
    View to accept CookieGroup.
    """

    def process(self, request, response, varname):
        accept_cookies(request, response, varname)


class CookieGroupDeclineView(CookieGroupBaseProcessView):
    """
    View to decline CookieGroup.
    """

    def process(self, request, response, varname):
        decline_cookies(request, response, varname)

    def delete(self, request, *args, **kwargs):
        return self.post(request, *args, **kwargs)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django import template
try:
    from django.urls import reverse
except ImportError:
    from django.core.urlresolvers import reverse

from cookie_consent.util import (
    get_accepted_cookies,
    get_cookie_string,
    get_cookie_value_from_request,
    get_cookie_dict_from_request,
    are_all_cookies_accepted,
    get_not_accepted_or_declined_cookie_groups,
    is_cookie_consent_enabled,
)
from cookie_consent.conf import settings


register = template.Library()


@register.filter
def cookie_group_accepted(request, arg):
    """
    Filter returns if cookie group is accepted.

    Examples:
    ::

        {{ request|cookie_group_accepted:"analytics" }}
        {{ request|cookie_group_accepted:"analytics=*:.google.com" }}
    """
    value = get_cookie_value_from_request(request, *arg.split("="))
    return value is True


@register.filter
def cookie_group_declined(request, arg):
    """
    Filter returns if cookie group is declined.
    """
    value = get_cookie_value_from_request(request, *arg.split("="))
    return value is False


@register.filter
def all_cookies_accepted(request):
    """
    Filter returns if all cookies are accepted.
    """
    return are_all_cookies_accepted(request)


@register.simple_tag
def not_accepted_or_declined_cookie_groups(request):
    """
    Assignement tag returns cookie groups that does not yet given consent
    or decline.
    """
    return get_not_accepted_or_declined_cookie_groups(request)


@register.filter
def cookie_consent_enabled(request):
    """
    Filter returns if cookie consent enabled for this request.
    """
    return is_cookie_consent_enabled(request)


@register.simple_tag
def cookie_consent_accept_url(cookie_groups):
    """
    Assignement tag returns url for accepting given concept groups.
    """
    varnames = ",".join([g.varname for g in cookie_groups])
    url = reverse("cookie_consent_accept", kwargs={"varname": varnames})
    return url


@register.simple_tag
def cookie_consent_decline_url(cookie_groups):
    """
    Assignement tag returns url for declining given concept groups.
    """
    varnames = ",".join([g.varname for g in cookie_groups])
    url = reverse("cookie_consent_decline", kwargs={"varname": varnames})
    return url


@register.simple_tag
def get_accept_cookie_groups_cookie_string(request, cookie_groups):
    """
    Tag returns accept cookie string suitable to use in javascript.
    """
    cookie_dic = get_cookie_dict_from_request(request)
    for cookie_group in cookie_groups:
        cookie_dic[cookie_group.varname] = cookie_group.get_version()
    return get_cookie_string(cookie_dic)


@register.simple_tag
def get_decline_cookie_groups_cookie_string(request, cookie_groups):
    """
    Tag returns decline cookie string suitable to use in javascript.
    """
    cookie_dic = get_cookie_dict_from_request(request)
    for cookie_group in cookie_groups:
        cookie_dic[cookie_group.varname] = settings.COOKIE_CONSENT_DECLINE
    return get_cookie_string(cookie_dic)


@register.simple_tag
def js_type_for_cookie_consent(request, varname, cookie=None):
    """
    Tag returns "x/cookie_consent" when processing javascript
    will create an cookie and consent does not exists yet.

    Example::

      <script type="{% js_type_for_cookie_consent request "social" %}"
      data-varname="social">
        alert("Social cookie accepted");
      </script>
    """
    enabled = is_cookie_consent_enabled(request)
    if not enabled:
        res = True
    else:
        value = get_cookie_value_from_request(request, varname, cookie)
        if value is None:
            res = settings.COOKIE_CONSENT_OPT_OUT
        else:
            res = value
    return "text/javascript" if res else "x/cookie_consent"


@register.filter
def accepted_cookies(request):
    """
    Filter returns accepted cookies varnames.

    ::
        {{ request|accepted_cookies }}
    """
    return [c.varname for c in get_accepted_cookies(request)]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.conf.urls import url
from django.views.decorators.csrf import csrf_exempt

from .views import (
    CookieGroupListView,
    CookieGroupAcceptView,
    CookieGroupDeclineView,
)

urlpatterns = [
    url(r'^accept/$',
        csrf_exempt(CookieGroupAcceptView.as_view()),
        name='cookie_consent_accept_all'),
    url(r'^accept/(?P<varname>.*)/$',
        csrf_exempt(CookieGroupAcceptView.as_view()),
        name='cookie_consent_accept'),
    url(r'^decline/(?P<varname>.*)/$',
        csrf_exempt(CookieGroupDeclineView.as_view()),
        name='cookie_consent_decline'),
    url(r'^decline/$',
        csrf_exempt(CookieGroupDeclineView.as_view()),
        name='cookie_consent_decline_all'),
    url(r'^$',
        CookieGroupListView.as_view(),
        name='cookie_consent_cookie_group_list'),
]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.conf.urls import url

from .views import (
    TestPageView,
)

urlpatterns = [
    url(r'^$',
        TestPageView.as_view(),
        name='test_page'),
]



# Generated by Django 2.1 on 2019-02-08 14:14

import django.core.validators
from django.db import migrations, models
import django.db.models.deletion
import re


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='Cookie',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=250, verbose_name='Name')),
                ('description', models.TextField(blank=True, verbose_name='Description')),
                ('path', models.TextField(blank=True, default='/', verbose_name='Path')),
                ('domain', models.CharField(blank=True, max_length=250, verbose_name='Domain')),
                ('created', models.DateTimeField(auto_now_add=True, verbose_name='Created')),
            ],
            options={
                'verbose_name': 'Cookie',
                'verbose_name_plural': 'Cookies',
                'ordering': ['-created'],
            },
        ),
        migrations.CreateModel(
            name='CookieGroup',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('varname', models.CharField(max_length=32, validators=[django.core.validators.RegexValidator(re.compile('^[-_a-zA-Z0-9]+$'), "Enter a valid 'varname' consisting of letters, numbers, underscores or hyphens.", 'invalid')], verbose_name='Variable name')),
                ('name', models.CharField(blank=True, max_length=100, verbose_name='Name')),
                ('description', models.TextField(blank=True, verbose_name='Description')),
                ('is_required', models.BooleanField(default=False, help_text='Are cookies in this group required.', verbose_name='Is required')),
                ('is_deletable', models.BooleanField(default=True, help_text='Can cookies in this group be deleted.', verbose_name='Is deletable?')),
                ('ordering', models.IntegerField(default=0, verbose_name='Ordering')),
                ('created', models.DateTimeField(auto_now_add=True, verbose_name='Created')),
            ],
            options={
                'verbose_name': 'Cookie Group',
                'verbose_name_plural': 'Cookie Groups',
                'ordering': ['ordering'],
            },
        ),
        migrations.AddField(
            model_name='cookie',
            name='cookiegroup',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='cookie_consent.CookieGroup', verbose_name='Cookie Group'),
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

import re

from django.db import models
from django.utils.translation import ugettext_lazy as _
from django.core.validators import RegexValidator

from cookie_consent.cache import delete_cache


COOKIE_NAME_RE = re.compile(r'^[-_a-zA-Z0-9]+$')
validate_cookie_name = RegexValidator(
    COOKIE_NAME_RE,
    _(u"Enter a valid 'varname' consisting of letters, numbers"
      ", underscores or hyphens."),
    'invalid')


class CookieGroup(models.Model):
    varname = models.CharField(
        _('Variable name'),
        max_length=32,
        validators=[validate_cookie_name])
    name = models.CharField(_('Name'), max_length=100, blank=True)
    description = models.TextField(_('Description'), blank=True)
    is_required = models.BooleanField(
        _('Is required'),
        help_text=_('Are cookies in this group required.'),
        default=False)
    is_deletable = models.BooleanField(
        _('Is deletable?'),
        help_text=_('Can cookies in this group be deleted.'),
        default=True)
    ordering = models.IntegerField(_('Ordering'), default=0)
    created = models.DateTimeField(_('Created'), auto_now_add=True, blank=True)

    class Meta:
        verbose_name = _('Cookie Group')
        verbose_name_plural = _('Cookie Groups')
        ordering = ['ordering']

    def __str__(self):
        return self.name

    def get_version(self):
        try:
            return str(self.cookie_set.all()[0].get_version())
        except IndexError:
            return ""

    def delete(self, *args, **kwargs):
        super(CookieGroup, self).delete(*args, **kwargs)
        delete_cache()

    def save(self, *args, **kwargs):
        super(CookieGroup, self).save(*args, **kwargs)
        delete_cache()


class Cookie(models.Model):
    cookiegroup = models.ForeignKey(
        CookieGroup,
        verbose_name=CookieGroup._meta.verbose_name,
        on_delete=models.CASCADE)
    name = models.CharField(_('Name'), max_length=250)
    description = models.TextField(_('Description'), blank=True)
    path = models.TextField(_('Path'), blank=True, default="/")
    domain = models.CharField(_('Domain'), max_length=250, blank=True)
    created = models.DateTimeField(_('Created'), auto_now_add=True, blank=True)

    class Meta:
        verbose_name = _('Cookie')
        verbose_name_plural = _('Cookies')
        ordering = ['-created']

    def __str__(self):
        return "%s %s%s" % (self.name, self.domain, self.path)

    @property
    def varname(self):
        return "%s=%s:%s" % (self.cookiegroup.varname, self.name, self.domain)

    def get_version(self):
        return self.created.isoformat()

    def delete(self, *args, **kwargs):
        super(Cookie, self).delete(*args, **kwargs)
        delete_cache()

    def save(self, *args, **kwargs):
        super(Cookie, self).save(*args, **kwargs)
        delete_cache()


ACTION_ACCEPTED = 1
ACTION_DECLINED = -1
ACTION_CHOICES = (
    (ACTION_DECLINED, _('Declined')),
    (ACTION_ACCEPTED, _('Accepted')),
)


class LogItem(models.Model):
    action = models.IntegerField(_('Action'), choices=ACTION_CHOICES)
    cookiegroup = models.ForeignKey(
        CookieGroup,
        verbose_name=CookieGroup._meta.verbose_name,
        on_delete=models.CASCADE)
    version = models.CharField(_('Version'), max_length=32)
    created = models.DateTimeField(_('Created'), auto_now_add=True, blank=True)
    def __str__(self):
        return "%s %s" % (self.cookiegroup.name, self.version)

    class Meta:
        verbose_name = _('Log item')
        verbose_name_plural = _('Log items')
        ordering = ['-created']



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

import datetime

from django.utils.encoding import smart_str

from cookie_consent.cache import (
    get_cookie_group,
    all_cookie_groups,
    get_cookie,
)
from cookie_consent.models import (
    ACTION_ACCEPTED,
    ACTION_DECLINED,
    LogItem,
)
from cookie_consent.conf import settings


def parse_cookie_str(cookie):
    dic = {}
    if not cookie:
        return dic
    for c in cookie.split("|"):
        key, value = c.split("=")
        dic[key] = value
    return dic


def dict_to_cookie_str(dic):
    return "|".join(["%s=%s" % (k, v) for k, v in dic.items() if v])


def get_cookie_dict_from_request(request):
    cookie_str = request.COOKIES.get(settings.COOKIE_CONSENT_NAME)
    return parse_cookie_str(cookie_str)


def set_cookie_dict_to_response(response, dic):
    response.set_cookie(settings.COOKIE_CONSENT_NAME,
                        dict_to_cookie_str(dic),
                        settings.COOKIE_CONSENT_MAX_AGE)


def get_cookie_value_from_request(request, varname, cookie=None):
    """
    Returns if cookie group or its specific cookie has been accepted.

    Returns True or False when cookie is accepted or declined or None
    if cookie is not set.
    """
    cookie_dic = get_cookie_dict_from_request(request)
    if not cookie_dic:
        return None

    cookie_group = get_cookie_group(varname=varname)
    if not cookie_group:
        return None
    if cookie:
        name, domain = cookie.split(":")
        cookie = get_cookie(cookie_group, name, domain)
    else:
        cookie = None

    version = cookie_dic.get(varname, None)

    if version == settings.COOKIE_CONSENT_DECLINE:
        return False
    if version is None:
        return None
    if not cookie:
        v = cookie_group.get_version()
    else:
        v = cookie.get_version()
    if version >= v:
        return True
    return None


def get_cookie_groups(varname=None):
    if not varname:
        return all_cookie_groups().values()
    keys = varname.split(",")
    return [g for k, g in all_cookie_groups().items() if k in keys]


def accept_cookies(request, response, varname=None):
    """
    Accept cookies in Cookie Group specified by ``varname``.
    """
    cookie_dic = get_cookie_dict_from_request(request)
    for cookie_group in get_cookie_groups(varname):
        cookie_dic[cookie_group.varname] = cookie_group.get_version()
        LogItem.objects.create(action=ACTION_ACCEPTED,
                               cookiegroup=cookie_group,
                               version=cookie_group.get_version())
    set_cookie_dict_to_response(response, cookie_dic)


def delete_cookies(response, cookie_group):
    if cookie_group.is_deletable:
        for cookie in cookie_group.cookie_set.all():
            response.delete_cookie(smart_str(cookie.name),
                                   cookie.path, cookie.domain)


def decline_cookies(request, response, varname=None):
    """
    Decline and delete cookies in CookieGroup specified by ``varname``.
    """
    cookie_dic = get_cookie_dict_from_request(request)
    for cookie_group in get_cookie_groups(varname):
        cookie_dic[cookie_group.varname] = settings.COOKIE_CONSENT_DECLINE
        delete_cookies(response, cookie_group)
        LogItem.objects.create(action=ACTION_DECLINED,
                               cookiegroup=cookie_group,
                               version=cookie_group.get_version())
    set_cookie_dict_to_response(response, cookie_dic)


def are_all_cookies_accepted(request):
    """
    Returns if all cookies are accepted.
    """
    return all([get_cookie_value_from_request(request, cookie_group.varname)
                for cookie_group in get_cookie_groups()])


def get_not_accepted_or_declined_cookie_groups(request):
    """
    Returns all cookie groups that are neither accepted or declined.
    """
    return [cookie_group for cookie_group in get_cookie_groups()
            if get_cookie_value_from_request(
                request, cookie_group.varname) is None]


def is_cookie_consent_enabled(request):
    """
    Returns if django-cookie-consent is enabled for given request.
    """
    enabled = settings.COOKIE_CONSENT_ENABLED
    if callable(enabled):
        return enabled(request)
    else:
        return enabled


def get_cookie_string(cookie_dic):
    """
    Returns cookie in format suitable for use in javascript.
    """
    expires = datetime.datetime.now() + datetime.timedelta(
        seconds=settings.COOKIE_CONSENT_MAX_AGE)
    cookie_str = "%s=%s; expires=%s; path=/" % (
        settings.COOKIE_CONSENT_NAME,
        dict_to_cookie_str(cookie_dic),
        expires.strftime("%a, %d %b %Y %H:%M:%S GMT")
    )
    return cookie_str


def get_accepted_cookies(request):
    """
    Returns all accepted cookies.
    """
    cookie_dic = get_cookie_dict_from_request(request)
    accepted_cookies = []
    for cookie_group in all_cookie_groups().values():
        version = cookie_dic.get(cookie_group.varname, None)
        if not version or version == settings.COOKIE_CONSENT_DECLINE:
            continue
        for cookie in cookie_group.cookie_set.all():
            if version >= cookie.get_version():
                accepted_cookies.append(cookie)
    return accepted_cookies



# -*- coding: utf-8 -*-
from __future__ import unicode_literals
from cookie_consent.conf import settings
try:
    from django.core.cache import get_cache
    cache = get_cache(settings.COOKIE_CONSENT_CACHE_BACKEND)
except ImportError:
    from django.core.cache import caches
    cache = caches[settings.COOKIE_CONSENT_CACHE_BACKEND]



CACHE_KEY = 'cookie_consent_cache'
CACHE_TIMEOUT = 60 * 60


def delete_cache():
    cache.delete(CACHE_KEY)


def all_cookie_groups():
    items = cache.get(CACHE_KEY)
    if items is None:
        from cookie_consent.models import CookieGroup
        qs = CookieGroup.objects.filter(is_required=False)
        qs = qs.prefetch_related('cookie_set')
        items = dict([(g.varname, g) for g in qs])
        cache.set(CACHE_KEY, items, CACHE_TIMEOUT)
    return items


def get_cookie_group(varname):
    return all_cookie_groups().get(varname)


def get_cookie(cookie_group, name, domain):
    for cookie in cookie_group.cookie_set.all():
        if cookie.name == name and cookie.domain == domain:
            return cookie
    return None



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.core.exceptions import ValidationError
from django.test import (
    TestCase,
)

from cookie_consent.models import (
    Cookie,
    CookieGroup,
    validate_cookie_name,
)


class CookieGroupTest(TestCase):

    def setUp(self):
        self.cookie_group = CookieGroup.objects.create(
            varname="optional",
            name="Optional",
        )
        self.cookie = Cookie.objects.create(
            cookiegroup=self.cookie_group,
            name="foo",
        )

    def test_get_version(self):
        self.assertEqual(
            self.cookie_group.get_version(),
            self.cookie.created.isoformat()
        )


class CookieTest(TestCase):

    def setUp(self):
        self.cookie_group = CookieGroup.objects.create(
            varname="optional",
            name="Optional",
        )
        self.cookie = Cookie.objects.create(
            cookiegroup=self.cookie_group,
            name="foo",
            domain=".example.com",
        )

    def test_varname(self):
        self.assertEqual(self.cookie.varname, "optional=foo:.example.com")


class ValidateCookieNameTest(TestCase):

    def test_valid(self):
        validate_cookie_name("_foo-bar")

    def test_invalid(self):
        invalid_names = (
            "space inside",
            "a!b",
            "$",
        )
        for name in invalid_names:
            with self.assertRaises(ValidationError):
                validate_cookie_name("no spaces")



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.utils.encoding import smart_str

from cookie_consent.cache import all_cookie_groups
from cookie_consent.util import (
    get_cookie_dict_from_request,
    is_cookie_consent_enabled,
)
from cookie_consent.conf import settings


class CleanCookiesMiddleware(object):
    """
    Clean declined cookies or non accepted cookies if not
    COOKIE_CONSENT_OPT_OUT set.
    """
    def __init__(self, get_response):
        self.get_response = get_response

    def process_response(self, request, response):
        if not is_cookie_consent_enabled(request):
            return response
        cookie_dic = get_cookie_dict_from_request(request)
        for cookie_group in all_cookie_groups().values():
            if not cookie_group.is_deletable:
                continue
            group_version = cookie_dic.get(cookie_group.varname, None)
            for cookie in cookie_group.cookie_set.all():
                if cookie.name not in request.COOKIES:
                    continue
                if group_version == settings.COOKIE_CONSENT_DECLINE:
                    response.delete_cookie(smart_str(cookie.name),
                                           cookie.path, cookie.domain)
                if group_version < cookie.get_version() and not settings.COOKIE_CONSENT_OPT_OUT:
                    response.delete_cookie(
                        smart_str(cookie.name),
                        cookie.path, cookie.domain
                    )
        return response



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from datetime import datetime

from django.test import (
    TestCase,
)
from django.test.client import RequestFactory
from django.test.utils import override_settings

from cookie_consent.models import (
    Cookie,
    CookieGroup,
)
from cookie_consent.util import (
    get_accepted_cookies,
    get_cookie_value_from_request,
    get_cookie_groups,
    parse_cookie_str,
    dict_to_cookie_str,
    is_cookie_consent_enabled,
)
from cookie_consent.conf import settings


class UtilTest(TestCase):

    def setUp(self):
        self.cookie_group = CookieGroup.objects.create(
            varname="optional",
            name="Optional",
        )
        self.cookie = Cookie.objects.create(
            cookiegroup=self.cookie_group,
            name="foo",
        )
        self.factory = RequestFactory()
        self.request = self.factory.get('')

    def test_parse_cookie_str(self):
        cookie_str = "foo=2013-06-04T01:08:58.262162|bar=2013-06-04T01:08:58"
        res = parse_cookie_str(cookie_str)
        dic = {
            'foo': "2013-06-04T01:08:58.262162",
            'bar': "2013-06-04T01:08:58",
        }
        self.assertEqual(res, dic)

    def test_dict_to_cookie_str(self):
        cookie_str = "|"
        dic = {
            'foo': "2013-06-04T01:08:58.262162",
            'bar': "2013-06-04T01:08:58",
        }
        cookie_str = dict_to_cookie_str(dic)
        self.assertEqual(parse_cookie_str(cookie_str), dic)

    def test_get_cookie_value_from_request(self):
        cookie_str = dict_to_cookie_str({
            "optional": self.cookie_group.get_version()
        })
        self.request.COOKIES[settings.COOKIE_CONSENT_NAME] = cookie_str
        res = get_cookie_value_from_request(self.request, 'optional')
        self.assertTrue(res)

    def test_get_cookie_value_from_request_declined(self):
        cookie_str = dict_to_cookie_str({
            "optional": datetime(1999, 1, 1).isoformat()
        })
        self.request.COOKIES[settings.COOKIE_CONSENT_NAME] = cookie_str
        res = get_cookie_value_from_request(self.request, 'optional')
        self.assertFalse(res)

    def test_get_cookie_value_from_request_empty(self):
        res = get_cookie_value_from_request(self.request, 'optional')
        self.assertIsNone(res)

    def test_get_cookie_value_from_request_added_cookies(self):
        cookie_str = dict_to_cookie_str({
            "optional": self.cookie_group.get_version(),
        })
        Cookie.objects.create(
            cookiegroup=self.cookie_group,
            name="bar",
            domain=".example.com",
        )
        self.request.COOKIES[settings.COOKIE_CONSENT_NAME] = cookie_str
        res = get_cookie_value_from_request(self.request, 'optional')
        self.assertIsNone(res)

    def test_get_cookie_value_from_request_specific_cookie(self):
        cookie_str = dict_to_cookie_str({
            "optional": self.cookie_group.get_version()
        })
        self.request.COOKIES[settings.COOKIE_CONSENT_NAME] = cookie_str
        res = get_cookie_value_from_request(self.request, 'optional', "foo:")
        self.assertTrue(res)

        Cookie.objects.create(
            cookiegroup=self.cookie_group,
            name="bar",
            domain=".example.com",
        )
        res = get_cookie_value_from_request(self.request, 'optional',
                                            "bar:.example.com")
        self.assertFalse(res)

        res = get_cookie_value_from_request(self.request, 'optional', "foo:")
        self.assertTrue(res)

        cookie_str = dict_to_cookie_str({
            "optional": self.cookie_group.get_version()
        })
        self.request.COOKIES[settings.COOKIE_CONSENT_NAME] = cookie_str
        res = get_cookie_value_from_request(self.request, 'optional',
                                            "bar:.example.com")
        self.assertTrue(res)

    def test_is_cookie_consent_enabled(self):
        self.assertTrue(is_cookie_consent_enabled(None))

    @override_settings(COOKIE_CONSENT_ENABLED=lambda r: False)
    def test_is_cookie_consent_enabled_callable(self):
        self.assertFalse(is_cookie_consent_enabled(None))

    def test_get_cookie_groups(self):
        self.assertIn(self.cookie_group, get_cookie_groups("optional"))

        cookie_group2 = CookieGroup.objects.create(
            varname="foo",
            name="foo",
        )
        self.assertIn(self.cookie_group, get_cookie_groups("foo,optional"))
        self.assertIn(cookie_group2, get_cookie_groups("foo,optional"))

    def test_get_accepted_cookies(self):
        cookie_str = dict_to_cookie_str({
            "optional": self.cookie_group.get_version()
        })
        self.request.COOKIES[settings.COOKIE_CONSENT_NAME] = cookie_str
        cookies = get_accepted_cookies(self.request)
        self.assertIn(self.cookie, cookies)



from distutils.core import setup
from setuptools import find_packages


version = '0.2.0'

CLASSIFIERS = [
    'Framework :: Django',
    'Intended Audience :: Developers',
    'License :: OSI Approved :: BSD License',
    'Operating System :: OS Independent',
    'Topic :: Software Development',
]

install_requires = [
    'Django>=1.9',
    'django-appconf',
]

setup(
    name="django-cookie-consent",
    description="Django cookie consent application",
    version=version,
    author="Informatika Mihelac",
    author_email="bmihelac@mihelac.org",
    url="https://github.com/bmihelac/django-cookie-consent",
    packages=find_packages(exclude=["tests"]),
    include_package_data=True,
    install_requires=install_requires,
    classifiers=CLASSIFIERS,
)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.test import (
    TestCase,
)
from django.test.utils import override_settings

from cookie_consent.models import (
    Cookie,
    CookieGroup,
)
from cookie_consent.cache import (
    get_cookie_group,
    get_cookie,
)


class CacheTest(TestCase):

    def setUp(self):
        self.cookie_group = CookieGroup.objects.create(
            varname="optional",
            name="Optional",
        )
        self.cookie = Cookie.objects.create(
            cookiegroup=self.cookie_group,
            name="foo",
        )

    def test_get_cookie_group(self):
        self.assertEqual(get_cookie_group("optional"), self.cookie_group)

    def test_get_cookie(self):
        cookie_group = get_cookie_group("optional")
        self.assertEqual(get_cookie(cookie_group, "foo", ""), self.cookie)

    def test_caching(self):
        CookieGroup.objects.create(
            varname="foo",
            name="Foo",
        )
        with self.assertNumQueries(2):
            cookie_group = get_cookie_group("optional")
            get_cookie_group("foo")
            get_cookie(cookie_group, "foo", "")

    def test_caching_expire(self):
        with self.assertNumQueries(2):
            cookie_group = get_cookie_group("optional")

        self.cookie_group.name = "Bar"
        self.cookie_group.save()

        with self.assertNumQueries(2):
            cookie_group = get_cookie_group("optional")
        self.assertEqual(cookie_group.name, "Bar")
